==> zego_uikit-1.0.1.tar.gz: lib/src/services/internal/core/zego_uikit_core_data.dart <==
part of 'zego_uikit_core.dart';

class ZegoUIKitCoreData {
  ZegoUIKitCoreUser localUser = ZegoUIKitCoreUser.localDefault();

  final Map<String, ZegoUIKitCoreUser> remoteUser = {}; // uid:user
  final Map<String, String> streamDic = {}; // stream_id:

  ZegoUIKitCoreRoom room = ZegoUIKitCoreRoom('');

  var userJoinStreamCtrl =
      StreamController<List<ZegoUIKitCoreUser>>.broadcast();
  var userLeaveStreamCtrl =
      StreamController<List<ZegoUIKitCoreUser>>.broadcast();
  var userListStreamCtrl =
      StreamController<List<ZegoUIKitCoreUser>>.broadcast();

  var audioVideoListStreamCtrl =
      StreamController<List<ZegoUIKitCoreUser>>.broadcast();

  ZegoUIKitVideoConfig videoConfig = ZegoUIKitVideoConfig();

  void clearStream() {
    remoteUser.forEach((_, user) {
      if (user.streamID.isNotEmpty) {
        stopPlayingStream(user.streamID);
      }
      user.destroyTextureRenderer();
    });
    if (localUser.streamID.isNotEmpty) {
      stopPublishingStream();
    }
    localUser.destroyTextureRenderer();
  }

  void clear() {
    clearStream();
    remoteUser.clear();
    streamDic.clear();
    room.clear();
  }

  ZegoUIKitCoreUser login(String id, String name) {
    localUser.id = id;
    localUser.name = name;

    userJoinStreamCtrl.add([localUser]);
    var userList = [
      localUser,
      ...remoteUser.entries.map((entry) => entry.value).toList()
    ];
    userListStreamCtrl.add(userList);

    return localUser;
  }

  void logout() {
    localUser.id = '';
    localUser.name = '';

    userLeaveStreamCtrl.add([localUser]);
    userListStreamCtrl
        .add(remoteUser.entries.map((entry) => entry.value).toList());
  }

  void setRoom(String roomID) {
    room = ZegoUIKitCoreRoom(roomID);
  }

  Future<void> startPreview() async {
    Future<void> _startPreview() async {
      assert(localUser.viewID != -1);
      ZegoCanvas previewCanvas = ZegoCanvas(
        localUser.viewID,
        viewMode: videoConfig.useVideoViewAspectFill
            ? ZegoViewMode.AspectFill
            : ZegoViewMode.AspectFit,
      );
      if (kIsWeb) {
        ZegoExpressEngine.instance.startPreview(canvas: previewCanvas);
      } else {
        ZegoExpressEngine.instance
          ..enableCamera(localUser.camera.value)
          ..startPreview(canvas: previewCanvas);
      }
    }

    createLocalUserVideoView(_startPreview);
  }

  Future<void> stopPreview() async {
    localUser.destroyTextureRenderer();

    ZegoExpressEngine.instance.stopPreview();
  }

  Future<void> startPublishingStream() async {
    if (localUser.streamID.isNotEmpty) {
      return;
    }

    localUser.streamID =
        generateStreamID(localUser.id, room.id, ZegoStreamType.main);
    streamDic[localUser.streamID] = localUser.id;

    Future<void> _startPublishStream() async {
      assert(localUser.viewID != -1);
      ZegoCanvas previewCanvas = ZegoCanvas(
        localUser.viewID,
        viewMode: videoConfig.useVideoViewAspectFill
            ? ZegoViewMode.AspectFill
            : ZegoViewMode.AspectFit,
      );
      if (kIsWeb) {
        ZegoExpressEngine.instance
          ..mutePublishStreamVideo(!localUser.camera.value)
          ..mutePublishStreamAudio(!!localUser.microphone.value)
          ..startPreview(canvas: previewCanvas)
          ..startPublishingStream(localUser.streamID).then((value) {
            onStreamDicChanged();
          });
      } else {
        ZegoExpressEngine.instance
          ..enableCamera(localUser.camera.value)
          ..muteMicrophone(!localUser.microphone.value)
          ..startPreview(canvas: previewCanvas)
          ..startPublishingStream(localUser.streamID).then((value) {
            onStreamDicChanged();
          });
      }
    }

    createLocalUserVideoView(_startPublishStream);
  }

  Future<void> stopPublishingStream() async {
    assert(localUser.streamID.isNotEmpty);

    streamDic.remove(localUser.streamID);
    localUser.streamID = "";
    localUser.destroyTextureRenderer();

    ZegoExpressEngine.instance
      ..stopPreview()
      ..stopPublishingStream().then((value) {
        onStreamDicChanged();
      });
  }

  Future<void> startPublishOrNot() async {
    if (room.id.isEmpty) {
      return;
    }
    if (localUser.camera.value || localUser.microphone.value) {
      startPublishingStream();
    } else {
      if (localUser.streamID.isNotEmpty) {
        stopPublishingStream();
      }
    }
  }

  void createLocalUserVideoView(VoidCallback onViewCreated) {
    if (kIsWeb) {
      if (-1 == localUser.viewID) {
        localUser.view.value =
            ZegoExpressEngine.instance.createPlatformView((viewID) {
          localUser.viewID = viewID;
          onViewCreated();
        });
      } else {
        //  user view had created
        onViewCreated();
      }
    } else {
      if (-1 == localUser.viewID) {
        ZegoExpressEngine.instance
            .createTextureRenderer(360, 640)
            .then((textureId) {
          localUser.viewID = textureId;
          localUser.view.value =
              Texture(key: ValueKey(localUser.id), textureId: textureId);
          onViewCreated();
        });
      } else {
        //  user view had created
        onViewCreated();
      }
    }
  }

  ZegoUIKitCoreUser removeUser(String uid) {
    ZegoUIKitCoreUser user = remoteUser.remove(uid)!;
    if (user.streamID.isNotEmpty) {
      stopPlayingStream(user.streamID);
    }
    return user;
  }

  Future<void> startPlayingStream(ZegoStream stream) async {
    streamDic[stream.streamID] = stream.user.userID;
    if (remoteUser[stream.user.userID] == null) {
      remoteUser[stream.user.userID] = ZegoUIKitCoreUser.fromZego(stream.user);
    }
    remoteUser[stream.user.userID]!.streamID = stream.streamID;

    if (kIsWeb) {
      remoteUser[stream.user.userID]!.view.value =
          ZegoExpressEngine.instance.createPlatformView((viewID) {
        remoteUser[stream.user.userID]!.viewID = viewID;
        ZegoCanvas canvas = ZegoCanvas(
          viewID,
          viewMode: videoConfig.useVideoViewAspectFill
              ? ZegoViewMode.AspectFill
              : ZegoViewMode.AspectFit,
        );
        ZegoExpressEngine.instance
            .startPlayingStream(stream.streamID, canvas: canvas);
      });
    } else {
      await ZegoExpressEngine.instance
          .createTextureRenderer(360, 640)
          .then((textureId) {
        remoteUser[stream.user.userID]!.view.value =
            Texture(key: ValueKey(stream.user.userID), textureId: textureId);
        remoteUser[stream.user.userID]!.viewID = textureId;
        ZegoCanvas canvas = ZegoCanvas(
          textureId,
          viewMode: videoConfig.useVideoViewAspectFill
              ? ZegoViewMode.AspectFill
              : ZegoViewMode.AspectFit,
        );
        ZegoExpressEngine.instance
            .startPlayingStream(stream.streamID, canvas: canvas);
      });
    }
  }

  void stopPlayingStream(String streamID) {
    assert(streamID.isNotEmpty);

    // stop playing stream
    ZegoExpressEngine.instance.stopPlayingStream(streamID);

    var targetUserID =
        streamDic.containsKey(streamID) ? streamDic[streamID] : "";
    if (remoteUser.containsKey(targetUserID)) {
      ZegoUIKitCoreUser user = remoteUser[targetUserID]!;
      assert(user.streamID == streamID);
      user.streamID = "";
      user.camera.value = false;
      user.microphone.value = false;
      user.soundLevel.add(0);
      user.destroyTextureRenderer();
    }

    // clear streamID
    streamDic.remove(streamID);
  }

  Future<void> onRoomStreamUpdate(String roomID, ZegoUpdateType updateType,
      List<ZegoStream> streamList, Map<String, dynamic> extendedData) async {
    if (updateType == ZegoUpdateType.Add) {
      for (final stream in streamList) {
        await startPlayingStream(stream);
      }
    } else {
      for (final stream in streamList) {
        stopPlayingStream(stream.streamID);
      }
    }

    onStreamDicChanged();
  }

  void onStreamDicChanged() {
    audioVideoListStreamCtrl.add(streamDic.entries
        .map((entry) {
          if (entry.value == localUser.id) {
            return localUser;
          }
          return remoteUser[entry.value]!;
        })
        .where((user) => user.view.value != null)
        .toList());
  }

  void onRoomUserUpdate(
      String roomID, ZegoUpdateType updateType, List<ZegoUser> userList) {
    if (updateType == ZegoUpdateType.Add) {
      for (final user in userList) {
        if (remoteUser.containsKey(user.userID)) {
          continue;
        }
        remoteUser[user.userID] = ZegoUIKitCoreUser.fromZego(user);
      }

      userJoinStreamCtrl.add(
          userList.map((user) => ZegoUIKitCoreUser.fromZego(user)).toList());
    } else {
      for (final user in userList) {
        removeUser(user.userID);
      }

      userLeaveStreamCtrl.add(
          userList.map((user) => ZegoUIKitCoreUser.fromZego(user)).toList());
    }

    userListStreamCtrl.add([
      localUser,
      ...remoteUser.entries.map((entry) => entry.value).toList()
    ]);
  }

  void onRemoteCameraStateUpdate(String streamID, ZegoRemoteDeviceState state) {
    ZegoUIKitCoreUser user = remoteUser[streamDic[streamID]!]!;
    switch (state) {
      case ZegoRemoteDeviceState.Open:
        user.camera.value = true;
        break;
      case ZegoRemoteDeviceState.Disable:
        user.camera.value = false;
        break;
      case ZegoRemoteDeviceState.Mute:
        user.camera.value = false;
        break;
      default:
        user.camera.value = false;
    }
  }

  void onRemoteMicStateUpdate(String streamID, ZegoRemoteDeviceState state) {
    ZegoUIKitCoreUser user = remoteUser[streamDic[streamID]!]!;
    switch (state) {
      case ZegoRemoteDeviceState.Open:
        user.microphone.value = true;
        break;
      default:
        user.microphone.value = false;
        user.soundLevel.add(0);
    }
  }

  void onRemoteSoundLevelUpdate(Map<String, double> soundLevels) {
    soundLevels.forEach((streamID, soundLevel) {
      if (!streamDic.containsKey(streamID)) {
        debugPrint("stream dic does not contain $streamID");
        return;
      }

      var targetUserID = streamDic[streamID]!;
      if (!remoteUser.containsKey(targetUserID)) {
        debugPrint("remote user does not contain $targetUserID");
        return;
      }

      ZegoUIKitCoreUser user = remoteUser[targetUserID]!;
      user.soundLevel.add(soundLevel);
    });
  }

  void onCapturedSoundLevelUpdate(double level) {
    localUser.soundLevel.add(level);
  }

  void onAudioRouteChange(ZegoAudioRoute audioRoute) {
    localUser.audioRoute.value = audioRoute;
  }

  void onPlayerVideoSizeChanged(String streamID, int width, int height) {
    ZegoUIKitCoreUser user = remoteUser[streamDic[streamID]!]!;
    log(" onPlayerVideoSizeChanged streamID: $streamID width: $width height: $height");
    Size size = Size(width.toDouble(), height.toDouble());
    if (user.viewSize.value != size) {
      user.viewSize.value = size;
    }
  }

  void onRoomStateChanged(String roomID, ZegoRoomStateChangedReason reason,
      int errorCode, Map<String, dynamic> extendedData) {
    log(" onRoomStateChanged roomID: $roomID, reason: $reason, errorCode: $errorCode, extendedData: $extendedData");
  }

  void onIMRecvCustomCommand(String roomID, ZegoUser fromUser, String command) {
    // var dict = jsonDecode(command) as Map<String, dynamic>;
  }
}

==> zego_uikit-1.3.0.tar.gz: lib/src/services/internal/core/zego_uikit_core_data.dart <==
part of 'zego_uikit_core.dart';

class ZegoUIKitCoreData {
  ZegoUIKitCoreUser localUser = ZegoUIKitCoreUser.localDefault();

  final List<ZegoUIKitCoreUser> remoteUsersList = [];

  final Map<String, String> streamDic = {}; // stream_id:user_id

  ZegoUIKitCoreRoom room = ZegoUIKitCoreRoom('');

  var userJoinStreamCtrl =
      StreamController<List<ZegoUIKitCoreUser>>.broadcast();
  var userLeaveStreamCtrl =
      StreamController<List<ZegoUIKitCoreUser>>.broadcast();
  var userListStreamCtrl =
      StreamController<List<ZegoUIKitCoreUser>>.broadcast();

  var audioVideoListStreamCtrl =
      StreamController<List<ZegoUIKitCoreUser>>.broadcast();

  ZegoUIKitVideoConfig videoConfig = ZegoUIKitVideoConfig();

  void clearStream() {
    for (var user in remoteUsersList) {
      if (user.streamID.isNotEmpty) {
        stopPlayingStream(user.streamID);
      }
      user.destroyTextureRenderer();
    }

    if (localUser.streamID.isNotEmpty) {
      stopPublishingStream();
    }
    localUser.destroyTextureRenderer();
  }

  void clear() {
    clearStream();

    remoteUsersList.clear();

    streamDic.clear();
    room.clear();
  }

  ZegoUIKitCoreUser login(String id, String name) {
    localUser.id = id;
    localUser.name = name;

    userJoinStreamCtrl.add([localUser]);
    var userList = [localUser, ...remoteUsersList];
    userListStreamCtrl.add(userList);

    return localUser;
  }

  void logout() {
    localUser.id = '';
    localUser.name = '';

    userLeaveStreamCtrl.add([localUser]);
    userListStreamCtrl.add(remoteUsersList);
  }

  void setRoom(String roomID) {
    room = ZegoUIKitCoreRoom(roomID);
  }

  Future<void> startPreview() async {
    Future<void> _startPreview() async {
      assert(localUser.viewID != -1);
      ZegoCanvas previewCanvas = ZegoCanvas(
        localUser.viewID,
        viewMode: videoConfig.useVideoViewAspectFill
            ? ZegoViewMode.AspectFill
            : ZegoViewMode.AspectFit,
      );
      if (kIsWeb) {
        ZegoExpressEngine.instance.startPreview(canvas: previewCanvas);
      } else {
        ZegoExpressEngine.instance
          ..enableCamera(localUser.camera.value)
          ..startPreview(canvas: previewCanvas);
      }
    }

    createLocalUserVideoView(_startPreview);
  }

  Future<void> stopPreview() async {
    localUser.destroyTextureRenderer();

    ZegoExpressEngine.instance.stopPreview();
  }

  Future<void> startPublishingStream() async {
    if (localUser.streamID.isNotEmpty) {
      return;
    }

    localUser.streamID =
        generateStreamID(localUser.id, room.id, ZegoStreamType.main);
    streamDic[localUser.streamID] = localUser.id;

    Future<void> _startPublishStream() async {
      assert(localUser.viewID != -1);
      ZegoCanvas previewCanvas = ZegoCanvas(
        localUser.viewID,
        viewMode: videoConfig.useVideoViewAspectFill
            ? ZegoViewMode.AspectFill
            : ZegoViewMode.AspectFit,
      );
      if (kIsWeb) {
        ZegoExpressEngine.instance
          ..mutePublishStreamVideo(!localUser.camera.value)
          ..mutePublishStreamAudio(!!localUser.microphone.value)
          ..startPreview(canvas: previewCanvas)
          ..startPublishingStream(localUser.streamID).then((value) {
            onStreamDicChanged();
          });
      } else {
        ZegoExpressEngine.instance
          ..enableCamera(localUser.camera.value)
          ..muteMicrophone(!localUser.microphone.value)
          ..startPreview(canvas: previewCanvas)
          ..startPublishingStream(localUser.streamID).then((value) {
            onStreamDicChanged();
          });
      }
    }

    createLocalUserVideoView(_startPublishStream);
  }

  Future<void> stopPublishingStream() async {
    assert(localUser.streamID.isNotEmpty);

    streamDic.remove(localUser.streamID);
    localUser.streamID = "";
    localUser.destroyTextureRenderer();

    ZegoExpressEngine.instance
      ..stopPreview()
      ..stopPublishingStream().then((value) {
        onStreamDicChanged();
      });
  }

  Future<void> startPublishOrNot() async {
    if (room.id.isEmpty) {
      return;
    }
    if (localUser.camera.value || localUser.microphone.value) {
      startPublishingStream();
    } else {
      if (localUser.streamID.isNotEmpty) {
        stopPublishingStream();
      }
    }
  }

  void createLocalUserVideoView(VoidCallback onViewCreated) {
    if (kIsWeb) {
      if (-1 == localUser.viewID) {
        localUser.view.value =
            ZegoExpressEngine.instance.createPlatformView((viewID) {
          localUser.viewID = viewID;
          onViewCreated();
        });
      } else {
        //  user view had created
        onViewCreated();
      }
    } else {
      if (-1 == localUser.viewID) {
        ZegoExpressEngine.instance
            .createTextureRenderer(360, 640)
            .then((textureId) {
          localUser.viewID = textureId;
          localUser.view.value =
              Texture(key: ValueKey(localUser.id), textureId: textureId);
          onViewCreated();
        });
      } else {
        //  user view had created
        onViewCreated();
      }
    }
  }

  ZegoUIKitCoreUser removeUser(String uid) {
    var targetIndex = remoteUsersList.indexWhere((user) => uid == user.id);
    if (-1 == targetIndex) {
      return ZegoUIKitCoreUser.empty();
    }

    var targetUser = remoteUsersList.removeAt(targetIndex);
    if (targetUser.streamID.isNotEmpty) {
      stopPlayingStream(targetUser.streamID);
    }
    return targetUser;
  }

  Future<void> startPlayingStream(ZegoStream stream) async {
    streamDic[stream.streamID] = stream.user.userID;

    var targetUserIndex =
        remoteUsersList.indexWhere((user) => stream.user.userID == user.id);
    if (-1 == targetUserIndex) {
      var targetUser = ZegoUIKitCoreUser.fromZego(stream.user);
      targetUser.streamID = stream.streamID;
      remoteUsersList.add(targetUser);

      targetUserIndex = remoteUsersList.length - 1;
    }
    remoteUsersList[targetUserIndex].streamID = stream.streamID;

    if (kIsWeb) {
      remoteUsersList[targetUserIndex].view.value =
          ZegoExpressEngine.instance.createPlatformView((viewID) {
        remoteUsersList[targetUserIndex].viewID = viewID;
        ZegoCanvas canvas = ZegoCanvas(
          viewID,
          viewMode: videoConfig.useVideoViewAspectFill
              ? ZegoViewMode.AspectFill
              : ZegoViewMode.AspectFit,
        );
        ZegoExpressEngine.instance
            .startPlayingStream(stream.streamID, canvas: canvas);
      });
    } else {
      await ZegoExpressEngine.instance
          .createTextureRenderer(360, 640)
          .then((textureId) {
        remoteUsersList[targetUserIndex].view.value =
            Texture(key: ValueKey(stream.user.userID), textureId: textureId);
        remoteUsersList[targetUserIndex].viewID = textureId;
        ZegoCanvas canvas = ZegoCanvas(
          textureId,
          viewMode: videoConfig.useVideoViewAspectFill
              ? ZegoViewMode.AspectFill
              : ZegoViewMode.AspectFit,
        );
        ZegoExpressEngine.instance
            .startPlayingStream(stream.streamID, canvas: canvas);
      });
    }
  }

  void stopPlayingStream(String streamID) {
    assert(streamID.isNotEmpty);

    // stop playing stream
    ZegoExpressEngine.instance.stopPlayingStream(streamID);

    var targetUserID =
        streamDic.containsKey(streamID) ? streamDic[streamID] : "";
    var targetUserIndex =
        remoteUsersList.indexWhere((user) => targetUserID == user.id);
    if (-1 != targetUserIndex) {
      ZegoUIKitCoreUser targetUser = remoteUsersList[targetUserIndex];
      assert(targetUser.streamID == streamID);
      targetUser.streamID = "";
      targetUser.camera.value = false;
      targetUser.microphone.value = false;
      targetUser.soundLevel.add(0);
      targetUser.destroyTextureRenderer();
    }

    // clear streamID
    streamDic.remove(streamID);
  }

  Future<void> onRoomStreamUpdate(String roomID, ZegoUpdateType updateType,
      List<ZegoStream> streamList, Map<String, dynamic> extendedData) async {
    if (updateType == ZegoUpdateType.Add) {
      for (final stream in streamList) {
        await startPlayingStream(stream);
      }
    } else {
      for (final stream in streamList) {
        stopPlayingStream(stream.streamID);
      }
    }

    onStreamDicChanged();
  }

  void onStreamDicChanged() {
    audioVideoListStreamCtrl.add(streamDic.entries
        .map((entry) {
          var targetUserID = entry.value;
          if (targetUserID == localUser.id) {
            return localUser;
          }
          return remoteUsersList.firstWhere((user) => targetUserID == user.id,
              orElse: () => ZegoUIKitCoreUser.empty());
        })
        .where((user) => user.id.isNotEmpty && user.view.value != null)
        .toList());
  }

  void onRoomUserUpdate(
      String roomID, ZegoUpdateType updateType, List<ZegoUser> userList) {
    if (updateType == ZegoUpdateType.Add) {
      for (final _user in userList) {
        var targetUserIndex =
            remoteUsersList.indexWhere((user) => _user.userID == user.id);
        if (-1 != targetUserIndex) {
          continue;
        }

        remoteUsersList.add(ZegoUIKitCoreUser.fromZego(_user));
      }

      userJoinStreamCtrl.add(
          userList.map((user) => ZegoUIKitCoreUser.fromZego(user)).toList());
    } else {
      for (final user in userList) {
        removeUser(user.userID);
      }

      userLeaveStreamCtrl.add(
          userList.map((user) => ZegoUIKitCoreUser.fromZego(user)).toList());
    }

    userListStreamCtrl.add([localUser, ...remoteUsersList]);
  }

  void onRemoteCameraStateUpdate(String streamID, ZegoRemoteDeviceState state) {
    var targetUserIndex =
        remoteUsersList.indexWhere((user) => streamDic[streamID]! == user.id);
    if (-1 == targetUserIndex) {
      debugPrint(
          "onRemoteCameraStateUpdate, stream user $streamID is not exist");
      return;
    }

    ZegoUIKitCoreUser targetUser = remoteUsersList[targetUserIndex];
    switch (state) {
      case ZegoRemoteDeviceState.Open:
        targetUser.camera.value = true;
        break;
      case ZegoRemoteDeviceState.Disable:
        targetUser.camera.value = false;
        break;
      case ZegoRemoteDeviceState.Mute:
        targetUser.camera.value = false;
        break;
      default:
        targetUser.camera.value = false;
    }
  }

  void onRemoteMicStateUpdate(String streamID, ZegoRemoteDeviceState state) {
    var targetUserIndex =
        remoteUsersList.indexWhere((user) => streamDic[streamID]! == user.id);
    if (-1 == targetUserIndex) {
      debugPrint("onRemoteMicStateUpdate, stream user $streamID is not exist");
      return;
    }

    ZegoUIKitCoreUser targetUser = remoteUsersList[targetUserIndex];
    switch (state) {
      case ZegoRemoteDeviceState.Open:
        targetUser.microphone.value = true;
        break;
      default:
        targetUser.microphone.value = false;
        targetUser.soundLevel.add(0);
    }
  }

  void onRemoteSoundLevelUpdate(Map<String, double> soundLevels) {
    soundLevels.forEach((streamID, soundLevel) {
      if (!streamDic.containsKey(streamID)) {
        debugPrint("stream dic does not contain $streamID");
        return;
      }

      var targetUserID = streamDic[streamID]!;
      var targetUserIndex =
          remoteUsersList.indexWhere((user) => targetUserID == user.id);
      if (-1 == targetUserIndex) {
        debugPrint("remote user does not contain $targetUserID");
        return;
      }

      remoteUsersList[targetUserIndex].soundLevel.add(soundLevel);
    });
  }

  void onCapturedSoundLevelUpdate(double level) {
    localUser.soundLevel.add(level);
  }

  void onAudioRouteChange(ZegoAudioRoute audioRoute) {
    localUser.audioRoute.value = audioRoute;
  }

  void onPlayerVideoSizeChanged(String streamID, int width, int height) {
    var targetUserIndex =
        remoteUsersList.indexWhere((user) => streamDic[streamID]! == user.id);
    if (-1 == targetUserIndex) {
      debugPrint(
          "onPlayerVideoSizeChanged, stream user $streamID is not exist");
      return;
    }

    ZegoUIKitCoreUser targetUser = remoteUsersList[targetUserIndex];
    log(" onPlayerVideoSizeChanged streamID: $streamID width: $width height: $height");
    Size size = Size(width.toDouble(), height.toDouble());
    if (targetUser.viewSize.value != size) {
      targetUser.viewSize.value = size;
    }
  }

  void onRoomStateChanged(String roomID, ZegoRoomStateChangedReason reason,
      int errorCode, Map<String, dynamic> extendedData) {
    log(" onRoomStateChanged roomID: $roomID, reason: $reason, errorCode: $errorCode, extendedData: $extendedData");
  }

  void onIMRecvCustomCommand(String roomID, ZegoUser fromUser, String command) {
    // var dict = jsonDecode(command) as Map<String, dynamic>;
  }
}
